using System;
using iNTERFACEWARE.Chameleon;
using iNTERFACEWARE.Iguana.Plugin;

namespace IguanaPlugin
{
   /// <summary>
   /// The SourcePluginExample is an example of an Iguana source plugin.
   /// </summary>
   class SourcePluginExample : Log
   {
      SourcePluginExample()
      {
         // Connect handlers to receive plugin events.
         source.MessageRequested += new Source.MessageRequestedEvent(OnMessageRequested);
         source.MessageRequestCompleted += new Source.MessageRequestCompletedEvent(OnMessageRequestCompleted);
         source.Disconnected += new Source.DisconnectedEvent(OnDisconnected);

         // The first step in making the generated PrefixTestEngine object useable 
         // is to load the message definition or 'vmd' file. This file contains
         // the information about the structure of the messages, tables, and 
         // data mappings that were defined in the Chameleon GUI. 
         //  
         // PrefixTestEngine is a generated engine which means that it has an interface
         // based on the Messages and Tables you define in your vmd file. The 
         // use of stub code is helpful since it means that the compiler can 
         // catch typos or spelling mistakes of fields at compile rather than 
         // run time. However, it is not the only way you can use Chameleon. It
         // is possible to use a non-stubcode interface for the Chameleon
         // engine which we call the 'dynamic interface'. Please review the 
         // manual documentation for more details if you need this functionality
         // - most customers find that the stub code interface works well for 
         // their applications.

         engine.AddLog(this);
         System.Console.WriteLine("Loading test.vmd...");
         engine.Load("test.vmd");
         engine.CurrentConfigIndex = 0;
         System.Console.WriteLine("test.vmd loaded");
      }

      /// <summary>
      /// Log interface implementation.
      /// </summary>
      public void Println(string line)
      {
         System.Console.WriteLine(line);
         logString += line;
         logString += "\n";
      }

      private string generateRandomMessage()
      {
         // Beginning of message generator block (comment used as an anchor for STC)
         switch (random.Next(0, 2))
         {				
            case 0:
               return (new GenerateDefault()).GenerateMessage(engine);
            case 1:
               return (new GeneratePatientRegister()).GenerateMessage(engine);
            default:
               return (new GenerateDefault()).GenerateMessage(engine);
         }
         // End of message generator block (comment used as an anchor for STC)
      }

      /// <summary>
      /// This event is triggered when Iguana requests a message.
      /// </summary>
      public void OnMessageRequested
      (
         object sender, 
         System.EventArgs args, 
         SourceMessageRequestedEventInputs inputs,
         SourceMessageRequestedEventOutputs outputs
      )
      {
         // Note that exceptions can never be thrown outside the handler.
         try 
         {
            try 
            {
               System.Console.WriteLine("");
               System.Console.WriteLine("*** Message Requested ***");

               // To simulate the behavior of a real-world plugin, we throttle the rate
               // at which we generate messages to at most once per second.
               //
               // Real world plugins need no such throttling mechanism.  Real-world plugins
               // should respond to message requests immediately with either the next available 
               // message, or an empty response if no messages are presently available.
               // 
               // When a plugin responds to a message request with an empty response (signifying
               // that no messages are presently available), Iguana will delay for the period 
               // specified by the channel's From Plugin component Polling Time setting before 
               // sending another message request.

               DateTime now = DateTime.Now;
               bool generateMessage = (now - lastMessageRequestTime).Seconds >= 1;
               lastMessageRequestTime = now;

               if (generateMessage) 
               {
                  // Set OutboundMessage to next available message.
                  logString = "";
                  String outboundMessage = generateRandomMessage();
                  outputs.OutboundMessage = outboundMessage;
                        
                  // Set PluginMessageId to a unique identifier for the message, which will
                  // be sent back to the plugin in the next messageRequestCompleted event.
                  String pluginMessageId = lastMessageRequestTime.ToString();
                  outputs.PluginMessageId = pluginMessageId;

                  System.Console.WriteLine("Message Generated (plugin message id: " + pluginMessageId + ")");
                  System.Console.WriteLine(outboundMessage.Replace('\r','\n'));
               } 
               else 
               {
                  System.Console.WriteLine("Responding with an empty message - no messages presently available.");
               }
            } 
            catch (Exception e) 
            {
               System.Console.WriteLine("Message Generation Error:");
               System.Console.WriteLine(e.ToString());

               // Add an error-level log message to Iguana's logs describing the failure.
               outputs.AddErrorLogMessage(e.ToString());

               // You could also use StopChannelFlag, StopChannelErrorFlag, and StopChannelReason 
               // to stop the Iguana channel with or without an error flag, and provide a 
               // reason for the stop request.
               //
               // outputs.setStopChannel(true);
               // outputs.setStopChannelErrorFlag(true);
               // outputs.setStopChannelReason("An error occurred.");
            } 

            // You can send additional error/warning/info/debug log messages to Iguana.
            // In this case we simply send an info-level log message containing the 
            // Chameleon engine's log output.
            if (logString.Length != 0) 
            {
               outputs.AddInfoLogMessage(logString);
            }
         } 
         catch (Exception e) 
         {
            System.Console.WriteLine(e.ToString());
         }
      }

      /// <summary>
      /// This event is received when Iguana has received the message sent via the previous MessageRequested event.
      /// </summary>
      public void OnMessageRequestCompleted
      (
         object sender, 
         System.EventArgs args, 
         SourceMessageRequestCompletedEventInputs inputs,
         SourceMessageRequestCompletedEventOutputs outputs
      )
      {
         // Note that exceptions can never be thrown outside the handler.
         try 
         {
            // PluginMessageId will contain the unique identifier provided by the plugin in the
            // previous call to messageRequested.
            //
            // If messageProcessed is false, Iguana was unable to enqueue the message, and it
            // should be sent again.

            System.Console.WriteLine("");
            System.Console.WriteLine("*** Message Request Completed (plugin message id: " + inputs.PluginMessageId + ") ***");
         } 
         catch (Exception e) 
         {
            System.Console.WriteLine(e.ToString());
         }
      }

      /// <summary>
      /// This event is triggered when the plugin disconnects from Iguana.
      /// </summary>
      public void OnDisconnected(object sender, System.EventArgs Args)
      {
         // Note that exceptions can never be thrown outside the handler.
         try 
         {
            System.Console.WriteLine("Source disconnected.");
         }
         catch (Exception e) 
         {
            System.Console.WriteLine(e.ToString());
         }
      }

      public Source source = new Source();
      private SourcePluginEngine engine = new SourcePluginEngine();
      private DateTime lastMessageRequestTime = DateTime.Now;
      private Random random = new Random();
      private string logString = "";

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main(string[] args)
      {
         try 
         {
            if (args.Length < 3) 
            {
               throw new Exception("Invalid command line arguments.");
            }

            String hostname  = args[0];
            int    port      = int.Parse(args[1]);
            String channel   = args[2];

            SourcePluginExample sourcePluginExample = new SourcePluginExample();
            sourcePluginExample.source.Connect(hostname, port, channel);

            // Wait for the plugin to disconnect.
            // Could also use waitForDisconnectWithTimeout() here.
            sourcePluginExample.source.WaitForDisconnect();

            // Print the reason for disconnection.
            DisconnectReason disconnectReason = sourcePluginExample.source.DisconnectReason;
            System.Console.WriteLine(disconnectReason.Description);
         } 
         catch (Exception e) 
         {
            System.Console.WriteLine(e.ToString());
            System.Console.WriteLine();
            System.Console.WriteLine("This is an Iguana source plugin.  It runs as a command line application");
            System.Console.WriteLine("which connects via TCP/IP to the plugin communication server within Iguana.");
            System.Console.WriteLine("It will be necessary to run a channel in Iguana and set up a channel with");
            System.Console.WriteLine("a From Plugin component for this to connect to.  See:");
            System.Console.WriteLine();
            System.Console.WriteLine("   http://www.interfaceware.com/manual/plugin_setup_csharp.html");
            System.Console.WriteLine();
            System.Console.WriteLine("Once the channel is set up and running you can connect to it with this usage:");
            System.Console.WriteLine();
            System.Console.WriteLine("  SourcePluginExample <iguana hostname> <plugin communication port> <channel name>");
            System.Console.WriteLine();
            System.Console.WriteLine("The plugin communication port is the port that Iguana listens on for plugin");
            System.Console.WriteLine("communication.");
            System.Threading.Thread.Sleep(10000);
         }
      }
   }
}
